@@ -15,13 +15,15 @@ def load_image(filename):
     """
     This method is used to load external resources, such as images.
     It is automatically called when resource added to document by `FPDF.image()`.
+    It always return a BytesIO buffer.
     """
     # if a bytesio instance is passed in, use it as is.
     if isinstance(filename, BytesIO):
         return filename
     # by default loading from network is allowed for all images
     if filename.startswith(("http://", "https://")):
-        with urlopen(filename) as url_file:
+        # disabling bandit rule as permitted schemes are whitelisted:
+        with urlopen(filename) as url_file:  # nosec B310
             return BytesIO(url_file.read())
     elif filename.startswith("data"):
         return _decode_base64_image(filename)
@@ -33,8 +35,7 @@ def _decode_base64_image(base64Image):
     "Decode the base 64 image string into an io byte stream."
     imageData = base64Image.split("base64,")[1]
     decodedData = base64.b64decode(imageData)
-    imageBytes = BytesIO(decodedData)
-    return imageBytes
+    return BytesIO(decodedData)
 
 
 def get_img_info(img, image_filter="AUTO", dims=None):
@@ -46,11 +47,11 @@ def get_img_info(img, image_filter="AUTO", dims=None):
     if not isinstance(img, Image.Image):
         img = Image.open(img)
     if dims:
-        img = img.resize(dims, resample=Image.ANTIALIAS)
+        img = img.resize(dims, resample=Image.Resampling.LANCZOS)
     if image_filter == "AUTO":
         # Very simple logic for now:
         image_filter = "DCTDecode" if img.format == "JPEG" else "FlateDecode"
-    if img.mode not in ["L", "LA", "RGB", "RGBA"]:
+    if img.mode not in ("L", "LA", "RGB", "RGBA"):
         img = img.convert("RGBA")
     w, h = img.size
     info = {}
