@@ -1,7 +1,8 @@
 import math
 import re
 from typing import NamedTuple
-import xml.etree.ElementTree
+
+from defusedxml.ElementTree import fromstring as parse_xml_str
 
 from . import drawing, html
 
@@ -21,8 +22,10 @@ _HANDY_NAMESPACES = {
     "xlink": "http://www.w3.org/1999/xlink",
 }
 
-ALPHABET = re.compile(r"([a-zA-Z])")
-NUMBER_SIGN = re.compile(r"([+-])")
+# "e" is excluded as it is used to build numbers like -2e-3:
+ALPHABET = re.compile(r"([a-df-zA-Z])")
+# We use a negative lookbehind to NOT split after "e" used as exponentiation:
+NUMBER_SIGN = re.compile(r"(?<!e)([+-])")
 NUMBER_SPLIT = re.compile(r"(?:\s+,\s+|\s+,|,\s+|\s+|,)")
 DECIMAL_DISASTER = re.compile(r"(\d+\.\d+|\d+\.|\.\d+)(\.)")
 TRANSFORM_GETTER = re.compile(
@@ -536,7 +539,7 @@ class SVGSmoothCubicCurve(NamedTuple):
             cls(c2=drawing.Point(x=c2x, y=c2y), end=drawing.Point(x=ex, y=ey))
         )
 
-    def render(self, path_gsds, style, last_item):
+    def render(self, path_gsds, style, last_item, initial_point):
         # technically, it would also be possible to chain on from a quadratic Bézier,
         # since we can convert those to cubic curves and then retrieve the appropriate
         # control point. However, the SVG specification states in
@@ -549,15 +552,19 @@ class SVGSmoothCubicCurve(NamedTuple):
             c1 = last_item.end_point
 
         return drawing.BezierCurve(c1, self.c2, self.end).render(
-            path_gsds, style, last_item
+            path_gsds, style, last_item, initial_point
         )
 
-    def render_debug(self, path_gsds, style, last_item, debug_stream, pfx):
+    def render_debug(
+        self, path_gsds, style, last_item, initial_point, debug_stream, pfx
+    ):
         # pylint: disable=unused-argument
-        rendered, resolved = self.render(path_gsds, style, last_item)
+        rendered, resolved, initial_point = self.render(
+            path_gsds, style, last_item, initial_point
+        )
         debug_stream.write(f"{self} resolved to {resolved}\n")
 
-        return rendered, resolved
+        return rendered, resolved, initial_point
 
 
 @force_nodocument
@@ -573,7 +580,7 @@ class SVGRelativeSmoothCubicCurve(NamedTuple):
             cls(c2=drawing.Point(x=c2x, y=c2y), end=drawing.Point(x=ex, y=ey))
         )
 
-    def render(self, path_gsds, style, last_item):
+    def render(self, path_gsds, style, last_item, initial_point):
         last_point = last_item.end_point
 
         if isinstance(last_item, drawing.BezierCurve):
@@ -584,14 +591,20 @@ class SVGRelativeSmoothCubicCurve(NamedTuple):
         c2 = last_point + self.c2
         end = last_point + self.end
 
-        return drawing.BezierCurve(c1, c2, end).render(path_gsds, style, last_item)
+        return drawing.BezierCurve(c1, c2, end).render(
+            path_gsds, style, last_item, initial_point
+        )
 
-    def render_debug(self, path_gsds, style, last_item, debug_stream, pfx):
+    def render_debug(
+        self, path_gsds, style, last_item, initial_point, debug_stream, pfx
+    ):
         # pylint: disable=unused-argument
-        rendered, resolved = self.render(path_gsds, style, last_item)
+        rendered, resolved, initial_point = self.render(
+            path_gsds, style, last_item, initial_point
+        )
         debug_stream.write(f"{self} resolved to {resolved}\n")
 
-        return rendered, resolved
+        return rendered, resolved, initial_point
 
 
 @force_nodocument
@@ -604,22 +617,26 @@ class SVGSmoothQuadraticCurve(NamedTuple):
     def from_path_points(cls, path, ex, ey):
         return path.add_path_element(cls(end=drawing.Point(x=ex, y=ey)))
 
-    def render(self, path_gsds, style, last_item):
+    def render(self, path_gsds, style, last_item, initial_point):
         if isinstance(last_item, drawing.QuadraticBezierCurve):
             ctrl = (2 * last_item.end) - last_item.ctrl
         else:
             ctrl = last_item.end_point
 
         return drawing.QuadraticBezierCurve(ctrl, self.end).render(
-            path_gsds, style, last_item
+            path_gsds, style, last_item, initial_point
         )
 
-    def render_debug(self, path_gsds, style, last_item, debug_stream, pfx):
+    def render_debug(
+        self, path_gsds, style, last_item, initial_point, debug_stream, pfx
+    ):
         # pylint: disable=unused-argument
-        rendered, resolved = self.render(path_gsds, style, last_item)
+        rendered, resolved, initial_point = self.render(
+            path_gsds, style, last_item, initial_point
+        )
         debug_stream.write(f"{self} resolved to {resolved}\n")
 
-        return rendered, resolved
+        return rendered, resolved, initial_point
 
 
 @force_nodocument
@@ -632,7 +649,7 @@ class SVGRelativeSmoothQuadraticCurve(NamedTuple):
     def from_path_points(cls, path, ex, ey):
         return path.add_path_element(cls(end=drawing.Point(x=ex, y=ey)))
 
-    def render(self, path_gsds, style, last_item):
+    def render(self, path_gsds, style, last_item, initial_point):
         last_point = last_item.end_point
 
         if isinstance(last_item, drawing.QuadraticBezierCurve):
@@ -643,15 +660,19 @@ class SVGRelativeSmoothQuadraticCurve(NamedTuple):
         end = last_point + self.end
 
         return drawing.QuadraticBezierCurve(ctrl, end).render(
-            path_gsds, style, last_item
+            path_gsds, style, last_item, initial_point
         )
 
-    def render_debug(self, path_gsds, style, last_item, debug_stream, pfx):
+    def render_debug(
+        self, path_gsds, style, last_item, initial_point, debug_stream, pfx
+    ):
         # pylint: disable=unused-argument
-        rendered, resolved = self.render(path_gsds, style, last_item)
+        rendered, resolved, initial_point = self.render(
+            path_gsds, style, last_item, initial_point
+        )
         debug_stream.write(f"{self} resolved to {resolved}\n")
 
-        return rendered, resolved
+        return rendered, resolved, initial_point
 
 
 path_directive_mapping = {
@@ -736,7 +757,10 @@ def svg_path_converter(pdf_path, svg_path):
                 {"m": "l", "M": "L"}[last_directive_name]
             ]
 
-        numbers, svg_path = _read_n_numbers(svg_path[read_idx:], read_count)
+        if read_count:
+            numbers, svg_path = _read_n_numbers(svg_path[read_idx:], read_count)
+        else:
+            numbers, svg_path = (), svg_path[read_idx:].strip()
 
         last_directive(pdf_path, *numbers)
 
@@ -766,7 +790,7 @@ class SVGObject:
     def __init__(self, svg_text):
         self.cross_references = {}
 
-        svg_tree = xml.etree.ElementTree.fromstring(svg_text)
+        svg_tree = parse_xml_str(svg_text)
 
         if svg_tree.tag not in xmlns_lookup("svg", "svg"):
             raise ValueError(f"root tag must be svg, not {svg_tree.tag}")
@@ -788,27 +812,21 @@ class SVGObject:
         else:
             self.preserve_ar = True
 
-        if width is None:
-            width = Percent(100)
-        else:
+        self.width = None
+        if width is not None:
             width.strip()
             if width.endswith("%"):
-                width = Percent(width[:-1])
+                self.width = Percent(width[:-1])
             else:
-                width = resolve_length(width)
+                self.width = resolve_length(width)
 
-        self.width = width
-
-        if height is None:
-            height = Percent(100)
-        else:
+        self.height = None
+        if height is not None:
             height.strip()
             if height.endswith("%"):
-                height = Percent(height[:-1])
+                self.height = Percent(height[:-1])
             else:
-                height = resolve_length(height)
-
-        self.height = height
+                self.height = resolve_length(height)
 
         if viewbox is None:
             self.viewbox = None
@@ -852,9 +870,11 @@ class SVGObject:
             The same thing as `SVGObject.transform_to_rect_viewport`.
         """
 
-        return self.transform_to_rect_viewport(pdf.k, pdf.w, pdf.h, align_viewbox)
+        return self.transform_to_rect_viewport(pdf.k, pdf.epw, pdf.eph, align_viewbox)
 
-    def transform_to_rect_viewport(self, scale, width, height, align_viewbox=True):
+    def transform_to_rect_viewport(
+        self, scale, width, height, align_viewbox=True, ignore_svg_top_attrs=False
+    ):
         """
         Size the converted SVG paths to an arbitrarily sized viewport.
 
@@ -869,6 +889,9 @@ class SVGObject:
             height (Number): the height of the viewport to scale to in document units.
             align_viewbox (bool): if True, mimic some of the SVG alignment rules if the
                 viewbox aspect ratio does not match that of the viewport.
+            ignore_svg_top_attrs (bool): ignore <svg> top attributes like "width", "height"
+                or "preserveAspectRatio" when figuring the image dimensions.
+                Require width & height to be provided as parameters.
 
         Returns:
             A tuple of (width, height, `fpdf.drawing.GraphicsContext`), where width and
@@ -878,20 +901,32 @@ class SVGObject:
             converted from the SVG, scaled to the given viewport size.
         """
 
-        if isinstance(self.width, Percent):
+        if ignore_svg_top_attrs:
+            vp_width = width
+        elif isinstance(self.width, Percent):
+            if not width:
+                raise ValueError(
+                    'SVG "width" is a percentage, hence a viewport width is required'
+                )
             vp_width = self.width * width / 100
         else:
-            vp_width = self.width
+            vp_width = self.width or width
 
-        if isinstance(self.height, Percent):
+        if ignore_svg_top_attrs:
+            vp_height = height
+        elif isinstance(self.height, Percent):
+            if not height:
+                raise ValueError(
+                    'SVG "height" is a percentage, hence a viewport height is required'
+                )
             vp_height = self.height * height / 100
         else:
-            vp_height = self.height
+            vp_height = self.height or height
 
-        if scale != 1:
-            transform = drawing.Transform.scaling(1 / scale)
-        else:
+        if scale == 1:
             transform = drawing.Transform.identity()
+        else:
+            transform = drawing.Transform.scaling(1 / scale)
 
         if self.viewbox:
             vx, vy, vw, vh = self.viewbox
@@ -902,7 +937,7 @@ class SVGObject:
             w_ratio = vp_width / vw
             h_ratio = vp_height / vh
 
-            if self.preserve_ar and (w_ratio != h_ratio):
+            if not ignore_svg_top_attrs and self.preserve_ar and (w_ratio != h_ratio):
                 w_ratio = h_ratio = min(w_ratio, h_ratio)
 
             transform = (
@@ -934,7 +969,6 @@ class SVGObject:
             debug_stream (io.TextIO): the stream to which rendering debug info will be
                 written.
         """
-
         _, _, path = self.transform_to_page_viewport(pdf)
 
         old_x, old_y = pdf.x, pdf.y
